<?php
/*
 * exercise.php - base class for all exercise types
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 * This class provides methods accessible by exercise-classes implementing a
 * unique type of exercise. The methods to be used by inheriting exercise
 * classes invoke internal methods to parse XML and return values encapsulated
 * by XML.
 *
 * @abstract
 * @author Erik Schmitt
 * @date 2003-02-25
 *
 * @todo inherition of class variables works not properly (in php). #cdeiwiks ???
 */

class Exercise {

    var $eval_mode;


    /**
     * Simple constructor gets environment variables.
     */

    function Exercise() {
            $this->g_Action         = getExternalParam('action');
            $this->g_OpenObject = 'openobject_'. strtolower(getExternalParam('action'));
    }

    function getEvalMode ()
    {
        return $this->eval_mode;
    }

    function setEvalMode ($eval_mode)
    {
        $this->eval_mode = $eval_mode;
    }

   /**
    *    XXX
    *
    *    @abstract
    *    @access public
    *    @returns array with int (from 0 to 100 ) as an % value and save t/f
    */

    function evaluate($exercise_id,$solution_id){
    }


  /**
   * Generates the HTML-Output of an exercise to represent
   * it to the tutor to coorect a solution.
   *
   * @param Number exercise_id the unique id of the exercise
   * @param XML-String $p_exerciseXML the definition of the exercise in XML
   * @param XML-String $p_solutionXML the former solution of the learner. Optional.
   */


        function genCorrection
                        ( $p_exercise_id
                        , $p_exercise_sheet_id
                        , $p_exerciseXML
                        , $p_exercise_name
                        , $p_solutionXML = ""
                        , $p_exercise_solution_id = ""
                        , $p_student_comment
                        , $p_solution
                        )
        {
                $msg = $this->printFormToGetAnswer   // genOutput
                        ( $p_exercise_id
                        , $p_exercise_sheet_id
                        , $p_exerciseXML
                        , $p_exercise_name
                        , $p_solutionXML
                        , $p_exercise_solution_id
                        , $p_student_comment
                        , $p_solution
                        );

                # Punkte anzeigen und &auml;ndern k&ouml;nnen.
                return $msg;
        }

        /**
    *   Returns on solution to an exercise
    *
    *    @abstract
    *    @access public
    *    @returns string
    */
    function getSolution($exercise_id,$solution_id){
    }

   /**
    *    Returns solution to an Exercise
    *
    *    @abstract
    *    @access public
    *    @returns string
    */

    function genSolution($exercise_id){
    }


    /**
    *   marks a solution as the one to select
    *
    *    @abstract
    *    @access public
    *    @returns string
    */

    function setSolution($exercise_id,$solution_id){
    }

      /**
    *  returns an statistic output.
    *
    *    @abstract
    *    @access public
    *    @returns string
    */
    function getStatistic($exercise_id,$course_id){
    }
    

        /**************************************************************************/
        /*
        /* XML-related functions
        /*
        /**************************************************************************/



        /**
        * Returns the dom-document object of a given XML-String.
        *
        * @author Erik Schmitt
        * @param String $p_xml_string the xml-data as a string
        * @return Object the dom-document object
        */
        
        function getDomXML($p_xml_string)
        {
                // suppress warning, error message and dump is given out
                // below in case of errors.
                $dom_exerciseXML = domxml_open_mem(utf8_encode($p_xml_string));
                
                // Attention! CDATA sections get lost with calling domxml_open_mem()!
                
                if ($dom_exerciseXML === FALSE)
                {
                        echo _("<hr />Error while parsing the document. Standard error message suppressed. Dump follows. Exit system.");
                        echo '<p />' . _("DUMP: ") . htmlentities($p_xml_string);
                        echo '<p />' . _("Exit.") . '<hr />';
                        exit;
                }
                return ($dom_exerciseXML);
        }



        /**
         * Returns the content of all tags looked up in an xml-string as an
         * array.
         *
         * Returns empty array if nothing was found.
         *
         * @author Erik Schmitt
         * @date 2003-02-23
         * @param String $p_xml_string the xml-data as a string
         * @param String $p_lookup_tag the tag to look up in the xml-string
         * @return Array $m_return_value the array with the content found
         */

        function getTagsFromXML($p_xml_string, $p_lookup_tag) {

                $m_return_value = array();

                if($p_xml_string != "") { // is empty when a new exerise is created
        
                        $m_exercise_dom_xml = $this->getDomXML($p_xml_string);
        
                        $m_exercise_elements = $m_exercise_dom_xml->get_elements_by_tagname($p_lookup_tag);
                
                        $i = 0;
                        foreach ($m_exercise_elements as $knod)
                        {
                                $m_return_value[$i++] = utf8_decode($knod->get_content());
                        }
                }
                return ($m_return_value);
        }



        /**
         * Returns the content of all tags looked up in an xml-string as an
         * array.
         *
         * Returns empty array if nothing was found.
         *
         * @author Philipp Hgelmeyer
         * @date 2004-06-29
         * @param String $p_xml_string the xml-data as a string
         * @param String $p_lookup_tag the tag to look up in the xml-string
         * @param String $p_attribute the attribute to look up in each tag
         * @return Array $m_return_value the array with the content found
         */

         function getAttributesFromXML($p_xml_string, $p_lookup_tag,$p_attribute) {

        $m_return_value = array();
                
        if($p_xml_string != "") { // is empty when a new exerise is created
                        $m_exercise_dom_xml = $this->getDomXML($p_xml_string);
            $m_exercise_elements = $m_exercise_dom_xml->get_elements_by_tagname($p_lookup_tag);
            $i = 0;
            foreach ($m_exercise_elements as $knod)
            {
                $m_return_value[$i++] = utf8_decode($knod->get_attribute($p_attribute));
            }
        }
        return ($m_return_value);
    }

} // end class
?>
