<?php
/*
 *  $Id: Param.php,v 1.2 2004/11/25 16:45:43 micha Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */

/**
* The Param class allows passing parameters having a default value of NULL as reference.
*
* @author Michael Aichler <aichler@mediacluster.de>
* @package creole.utils
*/
class Param
{
  var $ref;

  /**
  * Static function for saving a reference to variable @c $param.
  * Returns an object of type @c Param with its member attribute
  * @c ref pointing to @c param.
  *
  * The reference to @c param can be retrieved through @ref Param::get()
  * later on.
  *
  * <code>
  *   class Foo extends BaseFoo
  *   {
  *     function save($con = null)
  *     {
  *       // retrieve reference to $con
  *       $con =& Param::get($con);
  *
  *       if ($con)
  *       {
  *         // ...
  *       }
  *     }
  *   }
  *
  *   $con =& Propel::getConnection();
  *   $foo =& new Foo();
  *   // ...
  *   $foo->save(Param::set($con));
  * </code>
  *
  * @param mixed $val
  * @return Param
  */
  function set(&$param)
  {
    return new Param($param);
  }

  /**
  * Static function for retrieving a reference to a variable
  * stored in a @ref Param object (normally retrieved by
  * @ref Param::set() ).
  *
  * If parameter @c $param is not an object of type @c Param
  * than the parameter itself is returned.
  *
  * See @ref Param::set() for example code.
  *
  * @param mixed $param
  * @return mixed
  */
  function & get(&$param)
  {
    if (is_a($param, 'Param')) {
      return $param->ref;
    }

    if ($param === null) {
      return null;
    }

    return $param;

    /*
    * Note: the following does not work as of php 4.3.8:
    * return is_a($param, 'Param') ? $param->ref : $param;
    */
  }
  
  /**
  * Protected Param ctor.
  *
  * @param mixed $ref
  * @return void
  */
  function Param(&$ref)
  {
    $this->ref =& $ref;
  }
  
}

